<?php
namespace Teknisa\CADFIS\Controllers;

use Teknisa\CADFIS\Services\Ger00000CadastramentosAutomaticoService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Zeedhi\Framework\DTO\Response\Error; 
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\XLS\Exception;
use Zeedhi\Framework\DTO\Response;


use Teknisa\CADFIS\Controllers\TeknisaCrud;
use Teknisa\CADFIS\Factories;
use Teknisa\CADFIS\Strategies;
use Teknisa\CADFIS\Strategies\Context;
use Teknisa\CADFIS\Controllers\Import\XlsImportFile;
use \Doctrine\ORM\EntityManager;
use \Teknisa\Libs\Util\Address;
use \Teknisa\Libs\Util\NewCode;

class Ger00000CadastramentosAutomaticoController extends TeknisaCrud {
    
    protected $crudService;
    
    public function __construct($crudService, ParameterBag $parameterBag, Ger00000CadastramentosAutomaticoService $service, Factories\StrategiesFactory $strategiesFactory,$entityManager, Environment $environment, NewCode $newCode) {
        parent::__construct($crudService, $parameterBag);
        $this->service = $service;
        $this->crudService = $crudService;
        $this->strategiesFactory = $strategiesFactory;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->newCode = $newCode;
    }
    
    public function importXls(DTO\Request $request, DTO\Response $response){
        try{ 
            $row = $request->getRow();
            $table = $this->service->readXls($row);
            $rows = $this->service->formatXls($table);
            
            if($row['TABLE_NAME'] == 'PRODUTO'){
                $completedRows = $this->service->productCode($rows);
                $completedRows = $this->service->productGeneroCode($completedRows);
                $response->addDataSet(new DataSet('import', $completedRows));
            }else{
                $response->addDataSet(new DataSet('import', $rows));
            }
        }
        catch(Exception $e){
            $errors = $e->getErrors();
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error($e->getMessage(), 500)); 
        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function verificaPrimaryKey(DTO\Request $request, DTO\Response $response){
        try{
            $dataSet = $request->getDataSet()->getRows();
            
            $errors = $this->service->verificaDado($dataSet);
            
            $response->addDataSet(new DataSet('errors', $errors));

        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveImportXlsProduct(DTO\Request $request, DTO\Response $response){
        try{
            $dataSet = $this->crudService->getRowsToSave($request, 'produto');
            $dataSet = $this->service->atualizaCdproduto($dataSet);
			$this->service->validaArvProd($dataSet);
			
            $this->crudService->save('produto', $dataSet);

            $this->service->insereAgrupamentos($dataSet);
            
        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    /*
     * Implementações utilizadas pelas telas do Tecfood
     */
    
    public function setContext($tableName){
	    return new Context($this->strategiesFactory->getFactory($tableName));
	}
	
    public function importXlsTecfood(DTO\Request $request, DTO\Response $response){
        try {
            $xlsImport = new XlsImportFile;
            
            $row = $request->getRow();
            
            $xlsImport->setColumnNames($row['CAMPOS']);
            
            $rows = $xlsImport->importData($row);
            
            if (empty($rows))
                 throw new \Exception('FILE_EMPTY');

            $context = self::setContext($row['TABLE_NAME']);
            
            $context->validate($rows);
            
            $context->formatToView($rows);
            
            $response->addDataSet(new DataSet('import', $rows));
            
        } catch(XlsException $e) {
            $errors = $e->getErrors();
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error($e->getMessage(), 500));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveImport(DTO\Request $request, DTO\Response $response){
        try {
            $data = $request->getDataSet()->getRows()[0];
            
            $rows = $data['ROWS'];
            
            $tableName = $data['TABLE_NAME'];
            
            $context = self::setContext($tableName);
            
            $context->formatToSave($rows);
            
            $context->save($rows);
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    /**
     * Salva Conta Contabil
     * @param Data $data
     * @param Response $response
     */
    public function saveContaContabil(Data $dataset, Response $response){
        try {
            // Dados do Registro
            $dataset  = $dataset->getDataSet()->getRows();
            $this->service->saveContaContabil($dataset);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaImportacaoCliente($request, $response){
        $data = $request->getDataSet()->getRows();


        $erros = array();
        $dataSource =  $this->dataSourceNameToSave;
        forEach($data as $row){
            try{

                $this->entityManager->beginTransaction();
                $dataSet = new DataSet("data", [$row]);
                $requestCliente = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                
                
                if($row["CDPAISCOBR"]){
                    $row["CDPAISCOBR"] = $this->completaPaisCobrança($row);    
                }
                if($row["CDPAISENTR"]){
                    $row["CDPAISENTR"] = $this->completaPaisCobrança($row);     
                }
                if($row["CDMUNICIPIOCOBR"]){
                    $row["CDMUNICIPIOCOBR"] = $this->completaMunicipioCobrança($row);    
                }
                
                if($row["CDMUNICIPIOENTR"]){
                    $row["CDMUNICIPIOENTR"] = $this->completaMunicipioEntrega($row);    
                }
                
                $erroMsg = $this->validaCamposCliente($row);

                if($erroMsg != ""){
                    $response->setError(new Error($erroMsg, 500));  
                    throw $e;
                };
                
                $this->service->insertCliente($row);
                //  if (strlen($response->getError()) > 0) {
                //      throw $e;
                //  };
                
                $this->dataSourceNameToSave = 'endeclie';
                parent::save($requestCliente, $response);
                //  if (strlen($response->getError()) > 0) {
                //      throw $e;
                //  };
                
                if($row['IDTPENDECLIECOBR']){
                    $requestEnde = $this->salvaEndeCobr($row, $request);    
                    parent::save($requestEnde, $response);
                };
                //  if (strlen($response->getError()) > 0) {
                //     throw $e;
                //  };
                
                if($row['IDTPENDECLIEENTR']){
                    $requestEntr = $this->salvaEndeEntr($row, $request);
                    parent::save($requestEntr, $response);
                };
                //  if (strlen($response->getError()) > 0) {
                //      throw $e;
                //  };
                

                $this->entityManager->commit();
                
            }catch(\Exception $e){
                if($erroMsg != ""){
                    $response->setError(new Error($erroMsg, 500));
                }else if(strstr($e->getMessage(), 'ORA-01400') !== false){
                    $response->setError(new Error("Cliente preenchido incorretamente.",500));
                }else if(strstr($e->getMessage(), 'ORA-00001') !== false){
                    $response->setError(new Error("Impossível adicionar. Registro já existe.",500));
                }else{
                    $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
                }
                $erro = $response->getError()->getMessage();
                array_push($erros, array("IDREF" => $row["IDREF"], "DESCERROR" => $erro));
                $response->addDataSet(new DataSet('errors', $erros));
                $this->entityManager->rollback();

            }

        }
    }
    
    public function salvaEndeCobr($row, $request){
        
        $passed = $row;
        
        $passed['IDTPENDECLIE'] = $row['IDTPENDECLIECOBR'];
        $passed['CDMUNICIPIO'] = $row['CDMUNICIPIOCOBR'];
        $passed['CDPAIS'] = $row['CDPAISCOBR'];
        $passed['DSCOMPENDCLI'] = $row['DSCOMPENDCLICOBR'];
        $passed['DSEMAILCLIE'] = $row['DSEMAILCLIECOBR'];
        $passed['DSEMAILCLIENFE'] = $row['DSEMAILCLIENFECOBR'];
        $passed['DSENDECLIE'] = $row['DSENDECLIECOBR'];
        $passed['NMBAIRCLIE'] = $row['NMBAIRCLIECOBR'];
        $passed['NRCELULARCLI'] = $row['NRCELULARCLICOBR'];
        $passed['NRCEPCLIE'] = $row['NRCEPCLIECOBR'];
        $passed['NRTELRESICLI'] = $row['NRTELRESICLICOBR'];
        $passed['SGESTADO'] = $row['SGESTADOCOBR'];
        
        $dataSet = new DataSet("data", [$passed]);
        $requestEnde = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                
        return $requestEnde;
    }
    
    public function salvaEndeEntr($row, $request){
        
        $passed = $row;
        $passed['IDTPENDECLIE'] = $row['IDTPENDECLIEENTR'];
        $passed['CDMUNICIPIO'] = $row['CDMUNICIPIOENTR'];
        $passed['CDPAIS'] = $row['CDPAISENTR'];
        $passed['DSCOMPENDCLI'] = $row['DSCOMPENDCLIENTR'];
        $passed['DSEMAILCLIE'] = $row['DSEMAILCLIEENTR'];
        $passed['DSEMAILCLIENFE'] = $row['DSEMAILCLIENFEENTR'];
        $passed['DSENDECLIE'] = $row['DSENDECLIEENTR'];
        $passed['NMBAIRCLIE'] = $row['NMBAIRCLIEENTR'];
        $passed['NRCELULARCLI'] = $row['NRCELULARCLIENTR'];
        $passed['NRCEPCLIE'] = $row['NRCEPCLIEENTR'];
        $passed['NRTELRESICLI'] = $row['NRTELRESICLIENTR'];
        $passed['SGESTADO'] = $row['SGESTADOENTR'];
        
        $dataSet = new DataSet("data", [$passed]);
        $requestEntr = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
        return $requestEntr;
        
    }
    
    public function completaPaisCobrança($row){
        while(strlen($row["CDPAISCOBR"]) < 4){
            $row["CDPAISCOBR"] = "0".$row["CDPAISCOBR"];
        }
        return $row["CDPAISCOBR"];
        
    }
    
    public function completaMunicipioCobrança($row){
        while(strlen($row["CDMUNICIPIOCOBR"]) < 4){
            $row["CDMUNICIPIOCOBR"] = "0".$row["CDMUNICIPIOCOBR"];
        }
        return $row["CDMUNICIPIOCOBR"];
    }
    
    public function completaPaisEntrega($row){
        while(strlen($row["CDPAISENTR"]) < 4){
            $row["CDPAISENTR"] = "0".$row["CDPAISENTR"];
        }
        return $row["CDPAISENTR"];
        
    }

    public function completaMunicipioEntrega($row){
        while(strlen($row["CDMUNICIPIOENTR"]) < 4){
            $row["CDMUNICIPIOENTR"] = "0".$row["CDMUNICIPIOENTR"];
        }
        return $row["CDMUNICIPIOENTR"];
    }
    
    public function validaCamposCliente($row){
        
        $erroMsg = "";
        $erros = 1;
        
        if($row["NRINSJURCLIE"] == "Invalido"){
            $erroMsg .= $erros." - CNPJ/CPF Invalido (Campo: NRINSJURCLIE)  ";
            $erros ++;
        }

        if($row["NRINSJURCLIE"]!=preg_replace("/[^0-9]/", "", $row["NRINSJURCLIE"])){
            $erroMsg .= $erros." - CNPJ/CPF tem que ser preenchido apenas com numeros (Campo: NRINSJURCLIE) ";
            $erros ++;
              
        }

        if($row["NRCEPCLIE"]){
            $address = new Address($this->entityManager, $this->environment, $this->newCode);
            $retorno = $address->searchCEP($row["NRCEPCLIE"]);    
            if($retorno == false){
                $erroMsg .= $erros." - CEP inválido. (Campo: NRCEPCLIE) ";
                $erros ++;
            };
        };
        
        if($row["NRCEPCLIECOBR"]){
            $address = new Address($this->entityManager, $this->environment, $this->newCode);
            $retorno = $address->searchCEP($row["NRCEPCLIECOBR"]);    
            if($retorno == false){
                $erroMsg .= $erros." - CEP inválido. (Campo: NRCEPCLIECOBR) ";
                $erros ++;
                
            };
        };
        
        if($row["NRCEPCLIEENTR"]){
            $address = new Address($this->entityManager, $this->environment, $this->newCode);
            $retorno = $address->searchCEP($row["NRCEPCLIEENTR"]);    
            if($retorno == false){
                $erroMsg .= $erros." - CEP inválido. (Campo: NRCEPCLIEENTR) ";
                $erros ++;
                
            };
        };
        
        if(($row["IDTPIJURCLIE"] != 'F') and ($row["IDTPIJURCLIE"] != 'J') and ($row["IDTPIJURCLIE"] != 'I') and ($row["IDTPIJURCLIE"] != 'O')){
            $erroMsg .= $erros." - Tipo de inscrição inválido (Campo: IDTPIJURCLIE) ";
            $erros ++;
                
        };
        
        if(($row["IDENDEMISCLI"] != 'P') and ($row["IDENDEMISCLI"] != 'C') and ($row["IDENDEMISCLI"] != 'E') and ($row["IDENDEMISCLI"] != 'X') and ($row["IDENDEMISCLI"] != 'T')){
            $erroMsg .= $erros." - Tipo de endereço utilizado na emissão de documentos inválido (Campo: IDENDEMISCLI)   ";
            $erros ++;
            
        };
        
        if(($row["IDENDCFOPCLI"] != 'P') and ($row["IDENDCFOPCLI"] != 'C') and ($row["IDENDCFOPCLI"] != 'E') and ($row["IDENDCFOPCLI"] != 'X') and ($row["IDENDCFOPCLI"] != 'T')){
            $erroMsg .= $erros." - Tipo de endereço considerado na geração do CFOP inválido (Campo: IDENDCFOPCLI)   ";
            $erros ++;
            
        };
        
        if(($row["IDCLIEATIV"] != 'S') and ($row["IDCLIEATIV"] != 'N')){
            $erroMsg .= $erros." - Identificador de cliente ativo inválido (Campo: IDCLIEATIV)  ";
            $erros ++;
            
        };
        
        if(($row["IDPRIPUBCLIE"] != 'V') and ($row["IDPRIPUBCLIE"] != 'B')){
            $erroMsg .= $erros." - Identificador do tipo do cliente inválido (Campo: IDPRIPUBCLIE)  ";
            $erros ++;
            
        };
        
        if(($row["IDVERSALDCLI"] != 'S') and ($row["IDVERSALDCLI"] != 'N')){
            $erroMsg .= $erros." - Identificador de verificação do saldo do cliente inválido (Campo: IDVERSALDCLI)  ";
            $erros ++;
            
        };
        
        if(($row["IDUTSALDCLI"] != 'S') and ($row["IDUTSALDCLI"] != 'N')){
            $erroMsg .= $erros." - Identificador de utilização do saldo do cliente pelo consumidor inválido (Campo: IDUTSALDCLI)    ";
            $erros ++;
            
        };
        
        if(($row["IDCALCDVCLIE"] != '1') and ($row["IDCALCDVCLIE"] != '2') and ($row["IDCALCDVCLIE"] != '3')){
            $erroMsg .= $erros." - Indentificador de cálculo do dígito verificador do crachá inválido (Campo: IDCALCDVCLIE) ";
            $erros ++;
            
        };
        
        if(($row["IDUTSUBCLIEN"] != 'S') and ($row["IDUTSUBCLIEN"] != 'N')){
            $erroMsg .= $erros." - Identificador de utilização de subsídio inválido (Campo: IDUTSUBCLIEN)   ";
            $erros ++;
            
        };
        
        if($row["IDTPENDECLIE"] != 'P'){
            $erroMsg .= $erros." - Indentificador de endereço principal inválido (Campo: IDTPENDECLIE)  ";
            $erros ++;
            
        };
        
        if($row["IDTPENDECLIE"] == 'P'){
            if($this->service->validaEndereco($row)!=NULL){
            	$erroMsg .= $erros." - ".$this->service->validaEndereco($row);
            	$erros ++;
            }
        };
        
        if(($row["IDTPENDECLIECOBR"] != 'C') and ($row["IDTPENDECLIECOBR"] != '')){
            $erroMsg .= $erros." - Indentificador de endereço de cobrança inválido (Campo: IDTPENDECLIECOBR)    ";
            $erros ++;
            
        };
        
        if($row["IDTPENDECLIECOBR"] == 'C'){
            if($this->service->validaEnderecoCobr($row)!=NULL){
            $erroMsg .=  $erros." - ".$this->service->validaEnderecoCobr($row);
            $erros ++;
            };
        };
        
        if(($row["IDTPENDECLIEENTR"] != 'T') and ($row["IDTPENDECLIEENTR"] != '')){
            $erroMsg .= $erros." - Indentificador de endereço de entrega inválido (Campo: IDTPENDECLIEENTR) ";
            $erros ++;
            
        };

        if($row["IDTPENDECLIEENTR"] == 'T'){
            if($erroMsg .= $this->service->validaEnderecoEntr($row )!=NULL){
            $erroMsg .= $erros." - ".$this->service->validaEnderecoEntr($row );
            $erros ++;
            }
        };
        return $erroMsg;
        
    }
    
}